/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
* $Created by: HU Chi (huchinlp@foxmail.com) 2019-04-03
*/

#ifndef __DATASET_H__
#define __DATASET_H__

#include "../XGlobal.h"
#include "../XTensor.h"

#include <cstdio>
#include <fstream>
#include <unordered_map>
#include <vector>

using namespace std;

using BatchType = vector<vector<int>>;
using BucketType = vector<vector<int>>;
using BufferType = vector<vector<int>>;

namespace nts { // namespace nts(NiuTrans.Tensor)

/* A `DataSet` is associated with a file which contains variable length data.*/
class DataSet {

private:
    /* the data buffer */
    BufferType buffer;

    /* the pointer to file stream */
    ifstream* fp;

    /* number of different fields */
    size_t fieldNum;

    /* size of the data buffer */
    size_t bufferSize;

    /* size of used data in buffer */
    size_t bufferUsed;

    /* load data from a file to the buffer */
    void LoadDataToBuffer();

public:
    /* sort data in the buffer */
    virtual void Sort(){};

    /* modify data in the buffer */
    virtual void Process(){};

    /* group data to buckets */
    virtual BucketType Bucketing() { return BucketType(); };

    /* generate a mini-batch */
    void LoadBatch(XTensor& t, const int* indices, size_t batchSize, size_t field);

    /* constructor */
    explicit DataSet(const char* fname, size_t paraFieldNum, size_t paraBufferSize);
};

} // namespace nts(NiuTrans.Tensor)

#endif // __DATASET_H__