/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
* $Created by: HU Chi (huchinlp@foxmail.com) 2019-04-02
*/

#ifndef __EMBEDDING_H__
#define __EMBEDDING_H__

#include "../XGlobal.h"
#include "../XTensor.h"

#include <vector>
#include <cstdio>
#include <fstream>
#include <unordered_map>

using namespace std;
using namespace nts;

/* 
 * A `Embedding` is associated with an embeddings table(embedNum * embedDim).
 * The table can be loaded from a file or constructed from an instance.
 */
class Embedding {

public:
    XTensor embeddingTable;

public:
    /* save the embeddings table to a file */
    void Dump(const char* fname);

    /* looks up ids in a list of embedding tensors */
    XTensor EmbeddingLookup(const XTensor& ids);

    /* load an embeddings table from a file */
    explicit Embedding(const char* fname);

    /* construct table from an XTensor instance */
    explicit Embedding(XTensor& table);

    /* construct table from a array instance */
    explicit Embedding(const float* p, size_t embedNum, size_t embedDim);
};

#endif // __EMBEDDING_H__