/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
* $Created by: HU Chi (huchinlp@foxmail.com) 2019-04-03
*/

#ifndef __VOCABULARY_H__
#define __VOCABULARY_H__

#include "../XGlobal.h"
#include "../XTensor.h"

#include <cstdio>
#include <fstream>
#include <unordered_map>

#define LOG(...)                      \
    do {                              \
        fprintf(stderr, "[INFO] ");   \
        fprintf(stderr, __VA_ARGS__); \
        fprintf(stderr, "\n");        \
        fflush(stdout);               \
    } while (0)

#define ERR(...)                      \
    do {                              \
        fprintf(stderr, "[ERROR] ");  \
        fprintf(stderr, __VA_ARGS__); \
        fprintf(stderr, "\n");        \
        fflush(stdout);               \
    } while (0)

using namespace std;

namespace nts { // namespace nts(NiuTrans.Tensor)

/* 
 * A Vocabulary maps strings to integers, allowing for strings to be mapped to an out-of-vocabulary token.
 * Vocabularies are fit to a particular dataset, which we use to decide which tokens are in-vocabulary.
 * For convinience, a vocabulary can be built from a source file or be loaded from a vocabulary file.
 */
class Vocabulary {

private:
    /* current size of the vocabulary */
    int currentSize = 0;

    /* initialize mappings with some special tokens */
    void Init();

public:
    /* a dict maps tokens to indices */
    unordered_map<string, int> token2ID;

    /* a dict maps indices to tokens */
    unordered_map<int, string> id2Token;

    /* save the vocabulary to a file */
    void Dump(const char* vocabPath);

    /* append token to the vocabulary */
    void Insert(string token);

    /* maps integers to tokens */
    vector<string> ID2Token(vector<int> ids);

    /* maps tokens to integers */
    vector<int> Token2ID(vector<string> tokens);

	/* constructor */
	/* load the vocabulary from a file  */
    explicit Vocabulary(const char* vocabPath);

    /* built the vocabulary from the source file */
    explicit Vocabulary(const char* srcPath, int minFreq, int maxSize);

};

} // namespace nts(NiuTrans.Tensor)

#endif // __VOCABULARY_H__
