/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
* $Created by: HU Chi (huchinlp@foxmail.com) 2019-03-18
*/

#ifndef _T2T_DATA_READER_H_
#define _T2T_DATA_READER_H_

#include "../XTensor.h"
#include "DataReader.h"

#define LOG(...)                      \
    do {                              \
        fprintf(stderr, "[INFO] ");   \
        fprintf(stderr, __VA_ARGS__); \
        fprintf(stderr, "\n");        \
        fflush(stdout);               \
    } while (0)

#define ERR(...)                      \
    do {                              \
        fprintf(stderr, "[ERROR] ");  \
        fprintf(stderr, __VA_ARGS__); \
        fprintf(stderr, "\n");        \
        fflush(stdout);               \
    } while (0)

using namespace std;

namespace nts { // namespace nts(NiuTrans.Tensor)

class T2TDataSet : public DataSet {

public:
    /* source-side vocabulary */
    Vocabulary* srcVocab;

    /* target-side vocabulary */
    Vocabulary* tgtVocab;

    /* sequences buffer */
    vector<pair<vector<int>, vector<int>>> t2tDataBuf;

    /* constructor */
    explicit T2TDataSet(const char* fname, int paraBufferSize,
        const char* srcVocabPath, const char* tgtVocabPath, bool isShuffled)
        : DataSet(fname, paraBufferSize, isShuffled)
    {
        srcVocab = new Vocabulary(srcVocabPath);
        tgtVocab = new Vocabulary(tgtVocabPath);
    }

    /* destructor */
    ~T2TDataSet()
    {
        delete srcVocab;
        delete tgtVocab;
    }

    /* loading setences to the buffer */
    void LoadDataToBuf() override;

    /* post processing of sequences after loading */
    void Process() override;
};

class T2TDataManager : public DataManager {
public:

	/* the associated data set */
	T2TDataSet* t2tDataSet;

	/* constructor */
    explicit T2TDataManager(bool isShuffled, const char* fname,
		int paraBatchSize, int paraMaxSeqLen,
        int paraMinSeqLen, int paraBufferSize, 
		const char* srcVocabPath, const char* tgtVocabPath)
        : DataManager(fname, paraBufferSize, isShuffled, paraMaxSeqLen, paraMinSeqLen, paraBatchSize)
    {
        maxSeqLen = paraMaxSeqLen;
        minSeqLen = paraMinSeqLen;
        t2tDataSet = new T2TDataSet(fname, paraBufferSize, srcVocabPath, tgtVocabPath, isShuffled);
    }

	

	/*  */
    void LoadBatchMT(XTensor* batchEnc, XTensor* paddingEnc,
        XTensor* batchDec, XTensor* paddingDec,
        XTensor* label, int devID, XMem* mem,
        bool isTraining);
};

} // namespace nts(NiuTrans.Tensor)

#endif // _T2T_DATA_READER_H_
